@@ -33,7 +33,7 @@ class BipartiteBase(DataFrame):
 
         # Start logger
         logger_init(self)
-        self.logger.info('initializing BipartiteBase object')
+        # self.logger.info('initializing BipartiteBase object')
 
         if len(args) > 0 and isinstance(args[0], BipartiteBase): # Note that isinstance works for subclasses
             self.set_attributes(args[0])
@@ -86,7 +86,7 @@ class BipartiteBase(DataFrame):
             'str': 'str'
         }
 
-        self.logger.info('BipartiteBase object initialized')
+        # self.logger.info('BipartiteBase object initialized')
 
     @property
     def _constructor(self):
@@ -434,6 +434,12 @@ class BipartiteBase(DataFrame):
         frame = self.copy()
 
         frame.logger.info('beginning BipartiteBase data cleaning')
+
+        # First, correct columns
+        # Note this must be done before data_validity(), otherwise certain checks are not guaranteed to work
+        frame.logger.info('correcting columns')
+        frame.update_cols()
+
         frame.logger.info('checking quality of data')
         # Make sure data is valid - computes correct_cols, no_na, no_duplicates, connected, and contiguous, along with other checks (note that column names are corrected in data_validity() if all columns are in the data)
         BipartiteBase.data_validity(frame) # Shared data_validity
@@ -529,8 +535,13 @@ class BipartiteBase(DataFrame):
                     valid_types = to_list(frame.dtype_dict[frame.col_dtype_dict[col]])
                     if col_type not in valid_types:
                         frame.logger.info('{} has wrong dtype, should be {} but is {}'.format(frame.col_dict[subcol], frame.col_dtype_dict[col], col_type))
-                        col_dtypes = False
-                        cols = False
+                        if col in ['fid', 'wid', 'j']:
+                            frame.logger.info('{} has wrong dtype, converting to contiguous integers'.format(frame.col_dict[subcol]))
+                            frame.contiguous_ids(col)
+                        else:
+                            frame.logger.info('{} has wrong dtype. Please check that this is the correct column (it is supposed to give {}), and if it is, cast it to a valid datatype (these include: {})'.format(frame.col_dict[subcol], subcol, valid_types))
+                            col_dtypes = False
+                            cols = False
 
         frame.logger.info('column datatypes correct:' + str(col_dtypes))
         if not col_dtypes:
@@ -560,9 +571,9 @@ class BipartiteBase(DataFrame):
         frame.logger.info('columns correct:' + str(cols))
 
         if not cols:
-            frame.logger.info('correcting columns')
-            frame.update_cols()
-        frame.correct_cols = True
+            frame.correct_cols = False
+        else:
+            frame.correct_cols = True
 
         frame.logger.info('--- checking nan data ---')
         nans = frame.shape[0] - frame.dropna().shape[0]
@@ -664,13 +675,13 @@ class BipartiteBase(DataFrame):
 
     def drop_worker_year_duplicates(self, inplace=True):
         '''
-        Update data to include only the largest connected set of movers, and if firm ids are contiguous, also return the NetworkX Graph.
+        Keep only the highest paying job for worker-year duplicates.
 
         Arguments:
             inplace (bool): if True, modify in-place
 
         Returns:
-            frame (BipartiteBase): BipartiteBase that keeps only the highest pyaing job for worker-year duplicates. If no year columns, returns frame with no changes
+            frame (BipartiteBase): BipartiteBase that keeps only the highest paying job for worker-year duplicates. If no year columns, returns frame with no changes
         '''
         if inplace:
             frame = self
